# Generated by Django 4.2.2 on 2023-06-25 09:50

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("app", "0002_invoice_product"),
    ]

    operations = [
        migrations.CreateModel(
            name="InvoiceItem",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100)),
            ],
        ),
    ]
